-------- 1/ CRÉER UNE BASE DE DONNÉES DANS POSTGRESQL --------

-- Création de la base de données (à partir d'une autre base de données du serveur PostgreSQL)
DROP DATABASE IF EXISTS mci_webinar2;
CREATE DATABASE mci_webinar2
 WITH ENCODING='UTF8'
 OWNER=postgres
 CONNECTION LIMIT=25;

-- Se connecter à la base de données mci_webinar2
-- ex, avec la commande "psql -U postgres -d mci_webinar2" sur linux

-- Création d'un schéma 'mci'
DROP SCHEMA IF EXISTS mci CASCADE;
CREATE SCHEMA mci;

-------- 2/ CRÉER DES TABLES D’IMPORT ET IMPORTER LES DONNÉES --------

-- Création de la table base_imb
DROP TABLE IF EXISTS mci.base_imb;
CREATE TABLE mci.base_imb (
imb_id integer,
imb_longitude double precision,
imb_latitude double precision,
imb_code character varying(30),
imb_nbr_logloc integer,
imb_source character varying(5),
imb_type character varying(25),
imb_code_insee character varying(5),
addr_code character varying(30),
imb_num character varying(25),
addr_numero integer,
addr_rep character varying(6),
addr_nom_voie character varying(50),
addr_nom_ld character varying(50),
addr_code_insee character varying(5),
addr_nom_commune character varying(255),
addr_id_fantoir character varying(4),
addr_nom_afnor character varying(255), -- #Champ bêta pas présent dans la doc
addr_source character varying(5),
CONSTRAINT mci_base_imb_pkey PRIMARY KEY (imb_id) -- Clef primaire: impose que les imb_id soient uniques dans la table
);

-- Ajout d'indexes sur mci.base_imb
CREATE INDEX mci_base_imb_imb_id_idx ON mci.base_imb USING btree (imb_id);
CREATE INDEX mci_base_imb_imb_code_insee_idx ON mci.base_imb USING btree (imb_code_insee);

-- Copie des données du fichier base_imb.csv dans la table mci.base_imb
\COPY mci.base_imb from '/home/debian/test/webinar_2/base_imb_38.csv' delimiter ';' CSV header; --Mettre à jour selon le chemin de votre fichier

-- Création de la table d'éligiblité
DROP TABLE IF EXISTS mci.eligibilite;
CREATE TABLE mci.eligibilite (
imb_id integer, 
addr_code character varying(30), 
imb_code character varying(30), 
--code_operateur, --Pas encore présent dans l'open data bêta
code_techno character varying(4), 
classe_debit_montant character varying(6), 
classe_debit_descendant character varying(6), 
limitation integer, 
saturation character varying(1),
CONSTRAINT mci_base_imb_imb_id FOREIGN KEY (imb_id) REFERENCES mci.base_imb (imb_id) -- Clef étrangère: impose que les imb_id existent dans mci.base_imb
);

-- Ajout d'indexes
CREATE INDEX mci_eligibilite_imb_id_idx ON mci.eligibilite USING btree (imb_id);
CREATE INDEX mci_eligibilite_code_techno_idx ON mci.eligibilite USING btree (code_techno);
CREATE INDEX mci_eligibilite_classe_debit_descendant_idx ON mci.eligibilite USING btree (classe_debit_descendant);


-- Copie des données du fichier actuel.csv dans la table mci.eligibilite
\COPY mci.eligibilite from '/home/debian/test/webinar_2/actuel_38.csv' delimiter ';' CSV header; --Mettre à jour selon le chemin de votre fichier

-- Prévisualisation des données
SELECT * from mci.eligibilite limit 10;

-------- 4/ RÉALISEZ LE CALCUL --------

-- Créer une table pour reçevoir les résultats du calcul
DROP TABLE IF EXISTS mci.elig_bhd_cu;
CREATE TABLE mci.elig_bhd_cu(
code_insee character varying(5),
nbr integer,
log_bhd_cu integer
);

-- Ajouter dans la table les codes insee et les totaux de locaux par communes
INSERT INTO mci.elig_bhd_cu (code_insee,nbr)
SELECT imb_code_insee,sum(imb_nbr_logloc) 
FROM mci.base_imb GROUP BY imb_code_insee;

-- Calcul du taux d'éligiblité au 8 Mbit DSL par commune en Isère (1/2)
UPDATE mci.elig_bhd_cu SET log_bhd_cu = t_bhd_cu.log_bhd_cu 
FROM
	(SELECT sum(imb_nbr_logloc) as log_bhd_cu, imb_code_insee FROM mci.base_imb 
	WHERE EXISTS (select 1 from mci.eligibilite
				  where eligibilite.imb_id=base_imb.imb_id and eligibilite.code_techno='CU' and classe_debit_descendant in ('BHD8','THD30')) -- Les noms des classes de débit ont été modifiées
	GROUP BY imb_code_insee
	) AS t_bhd_cu
WHERE elig_bhd_cu.code_insee=t_bhd_cu.imb_code_insee
;

-- Ajout des valeurs à 0 pour les communes qui n'ont pas de 8 Mbit DSL cuivre (2/2)
UPDATE mci.elig_bhd_cu SET log_bhd_cu = 0 WHERE log_bhd_cu IS NULL;

-- Export de la table elig_bhd_cu
\COPY mci.elig_bhd_cu to '/home/debian/test/webinar_2/mci_elig_bhd_cu.csv' delimiter ';' CSV header;


--------- BONUS HORS WEBINAE --------

--------- 5/ PRÉREQUIS POUR RÉALISER DES CALCULS AVEC POSTGIS --------

-- Ajout de l'extension postgis
-- Non couvert dans ce tutoriel

-- Ajout d'une colonne de géometrie et création d'un champ geom (pour notamment pouvoir ensuite faire des calculs géographiques avec Postgis)
ALTER TABLE mci.base_imb ADD COLUMN geom geometry;

-- Calcul de la géometrie
UPDATE mci.base_imb set geom=ST_SetSRID(ST_MakePoint(imb_longitude,imb_latitude), 2154); -- 2154: le SRID si les coordonnées sont en Lambert 93